==> JavaGroups-0.9.3.zip: JavaGroups/Demos/Grid/Grid.java <==

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import JavaGroups.*;



/**
 * Application shows virtual synchrony. A thread in each process randomly chooses square
 * and computes a random color (red/green/blue tuple) for it. It then multicasts the
 * square's location (x,y) and new color to all members of the group. When the message is
 * received (method call SetColor), the corresponding square is set to the new color and
 * repainted.<p>
 * Note that a process must not update its local grid before or after multicasting the update as
 * this could lead to inconsistencies ! Therefore, the Grid demo requires loopback to be
 * enabled, i.e. multicasts to a groups always have to also be received by its sender !
 */

public class Grid extends Frame implements MessageListener, WindowListener, ActionListener, Runnable {
    private Channel                channel;
    private RpcDispatcher          dispatcher;
    private Panel                  main_panel, button_panel, bottom_panel;
    private Button                 start_button, stop_button, check_button;
    private Label                  checksum_label=new Label("Checksum: ");
    private final Font             default_font=new Font("Helvetica",Font.PLAIN,12);
    private String                 groupname="GridGroup";
    private boolean                started=false;
    private int                    NUM=4;
    private ColorComp[][]          grid;
    private Thread                 mythread;
    private Random                 random=new Random(System.currentTimeMillis());
    private int                    sleeptime=100;  // ms
    private OutputStream           log=null;



 
    public Grid(ChannelFactory factory, String properties, int num, int sleeptime, String log) {
	NUM=num;
	this.sleeptime=sleeptime;
	grid=new ColorComp[NUM][NUM];    
	main_panel=new Panel();
	main_panel.setLayout(new GridLayout(NUM, NUM));
	button_panel=new Panel();
	bottom_panel=new Panel();
	bottom_panel.setLayout(new BorderLayout());
	start_button=new Button("Start");
	start_button.setFont(default_font);
	start_button.addActionListener(this);


	try {
	    if(log != null)
		this.log=new FileOutputStream(log);
	}
	catch(Exception e) {
	    System.err.println(e);
	}
	
	stop_button=new Button("Stop");
	stop_button.setFont(default_font);
	stop_button.addActionListener(this);
	
	check_button=new Button("Checksum");
	check_button.setFont(default_font);
	check_button.addActionListener(this);

	button_panel.add("South", start_button);
	button_panel.add("South", stop_button);
	button_panel.add("South", check_button);
	
	bottom_panel.add("Center", checksum_label);
	bottom_panel.add("South", button_panel);
	checksum_label.setFont(default_font);

	setLayout(new BorderLayout());
	add("Center", main_panel);
	add("South",  bottom_panel);
	SetStarted(false);


	for(int x=0; x < NUM; x++) {
	    for(int y=0; y < NUM; y++) {
		grid[x][y]=new ColorComp(Color.blue);
		main_panel.add(grid[x][y]);
	    }
	}
	
	try {
	    channel=new JChannel(properties);
	    // channel.SetOpt(Channel.GET_STATE_EVENTS, new Boolean(true));
	    dispatcher=new RpcDispatcher(channel, this, null, this);
	    channel.Connect(groupname);
	    System.out.println(" -- Getting the initial state -");
	    channel.GetState(null, 0);
	    System.out.println(" -- done -");
	}
	catch(Exception e) {
	    System.err.println(e);
	}
    }

    
    void SetGrid(ColorComp[][] new_grid) {
	if(new_grid == null)
	    return;
	synchronized(grid) {
	    for(int i=0; i < NUM; i++)
		for(int j=0; j < NUM; j++)
		    grid[i][j].SetColor(new_grid[i][j].GetColor());
	}
    }

    
    private void Log(String msg) {
	if(log != null) {
	    try {
		log.write(msg.getBytes());
	    }
	    catch(Exception e) {
		System.err.println(e);
	    }
	}
    }



    public void Receive(Message msg) {}

    public Object GetState() {
	return grid.clone();
    }

    public void SetState(Object state) {
	if(state != null)
	    SetGrid((ColorComp[][])state);
    }




    private Color SelectColor() {
	int red=(Math.abs(random.nextInt()) % 255);
	int green=(Math.abs(random.nextInt()) % 255);
	int blue=(Math.abs(random.nextInt()) % 255);
	return new Color(red, green, blue);
    }



    public void run() {
	Color new_color;
	Integer x, y;
	Vector args=new Vector();

	while(started) {
	    x=new Integer(Math.abs(random.nextInt() % NUM));
	    y=new Integer(Math.abs(random.nextInt() % NUM));
	    new_color=SelectColor();

	    try {
		dispatcher.CallRemoteMethods(null, "SetColor", x, y, new_color, GroupRequest.GET_ALL, 0);
		Thread.currentThread().sleep(sleeptime);
	    }
	    catch(Exception e) {
		System.err.println(e);
	    }
	}
	SetStarted(false);
    }



	
    public void go() {
	try {
	    setSize(300,350);
 	    addWindowListener(this);
	    setVisible(true);
	    setTitle("Color Grid");
	    setBackground(Color.white);
	}
	catch(Exception e) {
	    System.err.println(e);
	    return;
	}
    }




    public static void main(String[] args) {
	ChannelFactory  factory=new JChannelFactory();
	Grid            grid=null;
	String          properties="UDP:PING:FD:STABLE:NAKACK:UNICAST:FRAG:FLUSH:GMS:VIEW_ENFORCER:"+ 
	                           "STATE_TRANSFER:QUEUE";
	String          arg, next_arg;
	int             num=4, sleeptime=100;
	String          logfile=null;
	String          gossip_host=null;
	int             gossip_port=0;

	try {
	    for(int i=0; i < args.length; i++) {
		arg=args[i];
		if(arg.equals("-type")) {
		    next_arg=args[++i];
		    if(next_arg.equals("ens")) {
			factory=new EnsChannelFactory();
			properties=null;
			continue;
		    }
		    else if(next_arg.equals("ibus")) {
			factory=(ChannelFactory)Class.forName("JavaGroups.IbusChannelFactory").newInstance();
			properties=null;
			continue;
		    }
		}
		else if(arg.equals("-ucast")) {
		    factory=new JChannelFactory();
		    properties="UDP(ip_mcast=false):GMS(gossip_host=" + args[++i] + 
			";gossip_port=" + new Integer(args[++i]).intValue() +")";
		}
		else if(arg.equals("-help")) {
		    System.out.println("Grid [-type ens | ibus ] " + 
				       "[-ucast <gossip host> <gossip port>] [-num <number>] "+
				       "[-sleep <time in ms>] [-log <filename>]");
		    return;
		}
		else if(arg.equals("-num")) {
		    num=new Integer(args[++i]).intValue();
		    continue;
		}		
		else if(arg.equals("-sleep")) {
		    sleeptime=new Integer(args[++i]).intValue();
		    continue;
		}
		else if(arg.equals("-log")) {
		    logfile=args[++i];
		    continue;
		}
	    }
	}
	catch(Exception e) {
	    System.out.println("Grid [-type ens | ibus] [-ucast <gossip host> <gossip port>] " +
			       "[-num <number>] [-sleep <time (in ms)>] " +
			       "[-log <filename>]");
	    return;
	}

	grid=new Grid(factory, properties, num, sleeptime, logfile);
	grid.go();
    }





    


    /* --------------- Callbacks --------------- */



    public void SetColor(Integer x, Integer y, Color new_color) {
	synchronized(grid) {
	    Log("SetColor  " + x + "," + y + " (" + new_color.getRed() + "|" + 
		new_color.getGreen() + "|" + new_color.getBlue() + ")\n");
	    grid[x.intValue()][y.intValue()].SetColor(new_color);
	}
    }


    public void Start() {
	if(started)
	    return;
	SetStarted(true);
	checksum_label.setText("Running");
	mythread=new Thread(this);	
	mythread.start();
    }


    void SetStarted(boolean flag) {
	started=flag;
	if(started) {
	    stop_button.setEnabled(true);
	    start_button.setEnabled(false);
	}
	else {
	    stop_button.setEnabled(false);
	    start_button.setEnabled(true);
	}
    }


    public void Stop() {
	if(started) {
	    SetStarted(false);
	    if(mythread != null) {
		try {
		    mythread.join();
		}
		catch(Exception e) {
		    System.err.println(e);
		}
		mythread.stop();
		mythread=null;
	    }	    
	    try {
		dispatcher.CallRemoteMethods(null, "Stop", GroupRequest.GET_ALL, 0);
	    }
	    catch(Exception e) {
		System.err.println(e);
	    }
	}
	/* Give other threads a chance to finish. If we compute the checksum and still
	   receive updates from other threads that haven't finished processing, our checksum
	   will not reflect the current grid ! A better implementation would use acks to
	   make sure all threads have ended ... */
	try {
	    Thread.currentThread().sleep(500);
	}
	catch(Exception e) {
	    System.err.println(e);
	}
	ComputeChecksum();
    }




    public void ComputeChecksum() {
	long   checksum=0;
	Color  col;
	synchronized(grid) {
	    for(int x=0; x < NUM; x++)
		for(int y=0; y < NUM; y++)
		    checksum+=grid[x][y].GetChecksum();
	}
	checksum_label.setText("Checksum: " + checksum);
    }



    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e) {
	setVisible(false);
	channel.Close();
	System.exit(0);
    }
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}


    public void actionPerformed(ActionEvent e) {
	String     command=e.getActionCommand();	
	if(command == "Start")
	    Start();
	else if(command == "Stop")
	    Stop();
	else if(command == "Checksum")
	    //ComputeChecksum();
	    System.out.println(channel.GetView());
	else
	    System.out.println("Unknown action");
    }


}

==> JavaGroups-0.9.6.zip: JavaGroups/Demos/Grid/Grid.java <==

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;
import JavaGroups.*;



/**
   Application shows virtual synchrony. A thread in each process randomly chooses a square
   and computes a random color (red/green/blue tuple) for it. It then multicasts the
   square's location (x,y) and new color to all members of the group. When the message is
   received (method call SetColor()), the corresponding square is set to the new color and
   repainted.<p>
   Note that a process must not update its local grid before or after multicasting the update as
   this could lead to inconsistencies ! Therefore, the Grid demo requires loopback to be
   enabled, i.e. multicasts to a groups always have to also be received by its sender !<p>
*/

public class Grid extends Frame implements MessageListener, WindowListener, ActionListener, Runnable {
    private Channel                channel;
    private RpcDispatcher          dispatcher;
    private Panel                  main_panel, button_panel, bottom_panel;
    private Button                 start_button, stop_button, check_button;
    private Label                  checksum_label=new Label("Checksum: ");
    private final Font             default_font=new Font("Helvetica",Font.PLAIN,12);
    private String                 groupname="GridGroup";
    private boolean                started=false;
    private int                    NUM=4;
    private ColorComp[][]          grid;
    private Thread                 mythread;
    private Random                 random=new Random(System.currentTimeMillis());
    private int                    sleeptime=100;  // ms
    private OutputStream           log=null;



 
    public Grid(ChannelFactory factory, String properties, int num, int sleeptime, String log) {
	NUM=num;
	this.sleeptime=sleeptime;
	grid=new ColorComp[NUM][NUM];    
	main_panel=new Panel();
	main_panel.setLayout(new GridLayout(NUM, NUM));
	button_panel=new Panel();
	bottom_panel=new Panel();
	bottom_panel.setLayout(new BorderLayout());
	start_button=new Button("Start");
	start_button.setFont(default_font);
	start_button.addActionListener(this);


	try {
	    if(log != null)
		this.log=new FileOutputStream(log);
	}
	catch(Exception e) {
	    System.err.println(e);
	}
	
	stop_button=new Button("Stop");
	stop_button.setFont(default_font);
	stop_button.addActionListener(this);
	
	check_button=new Button("Checksum");
	check_button.setFont(default_font);
	check_button.addActionListener(this);

	button_panel.add("South", start_button);
	button_panel.add("South", stop_button);
	button_panel.add("South", check_button);
	
	bottom_panel.add("Center", checksum_label);
	bottom_panel.add("South", button_panel);
	checksum_label.setFont(default_font);

	setLayout(new BorderLayout());
	add("Center", main_panel);
	add("South",  bottom_panel);
	SetStarted(false);


	for(int x=0; x < NUM; x++) {
	    for(int y=0; y < NUM; y++) {
		grid[x][y]=new ColorComp(Color.blue);
		main_panel.add(grid[x][y]);
	    }
	}
	
	try {
	    channel=new JChannel(properties);
	    // channel.SetOpt(Channel.GET_STATE_EVENTS, new Boolean(true));
	    dispatcher=new RpcDispatcher(channel, this, null, this);
	    channel.Connect(groupname);
	    System.out.println(" -- Getting the initial state -");
	    channel.GetState(null, 0);
	    System.out.println(" -- done -");
	}
	catch(Exception e) {
	    System.err.println(e);
	}
    }

    
    void SetGrid(ColorComp[][] new_grid) {
	if(new_grid == null)
	    return;
	synchronized(grid) {
	    for(int i=0; i < NUM; i++)
		for(int j=0; j < NUM; j++)
		    grid[i][j].SetColor(new_grid[i][j].GetColor());
	}
    }

    
    private void Log(String msg) {
	if(log != null) {
	    try {
		log.write(msg.getBytes());
	    }
	    catch(Exception e) {
		System.err.println(e);
	    }
	}
    }



    public void Receive(Message msg) {}

    public Object GetState() {
	return grid.clone();
    }

    public void SetState(Object state) {
	if(state != null)
	    SetGrid((ColorComp[][])state);
    }




    private Color SelectColor() {
	int red=(Math.abs(random.nextInt()) % 255);
	int green=(Math.abs(random.nextInt()) % 255);
	int blue=(Math.abs(random.nextInt()) % 255);
	return new Color(red, green, blue);
    }



    public void run() {
	Color new_color;
	Integer x, y;
	Vector args=new Vector();

	while(started) {
	    x=new Integer(Math.abs(random.nextInt() % NUM));
	    y=new Integer(Math.abs(random.nextInt() % NUM));
	    new_color=SelectColor();

	    try {
		dispatcher.CallRemoteMethods(null, "SetColor", x, y, new_color, GroupRequest.GET_ALL, 0);
		Thread.currentThread().sleep(sleeptime);
	    }
	    catch(Exception e) {
		System.err.println(e);
	    }
	}
	SetStarted(false);
    }



	
    public void go() {
	try {
	    setSize(300,350);
 	    addWindowListener(this);
	    setVisible(true);
	    setTitle("Color Grid");
	    setBackground(Color.white);
	}
	catch(Exception e) {
	    System.err.println(e);
	    return;
	}
    }




    public static void main(String[] args) {
	ChannelFactory  factory=new JChannelFactory();
	Grid            grid=null;
	String          properties="UDP:PING:FD:STABLE:NAKACK:UNICAST:FRAG:FLUSH:GMS:VIEW_ENFORCER:"+ 
	                           "STATE_TRANSFER:QUEUE";
	String          arg, next_arg;
	int             num=4, sleeptime=100;
	String          logfile=null;
	String          router_host=null;
	int             router_port=0;

	try {
	    for(int i=0; i < args.length; i++) {
		arg=args[i];
		if(arg.equals("-type")) {
		    next_arg=args[++i];
		    if(next_arg.equals("ens")) {
			factory=new EnsChannelFactory();
			properties=null;
			continue;
		    }
		    else if(next_arg.equals("ibus")) {
			factory=(ChannelFactory)Class.forName("JavaGroups.IbusChannelFactory").newInstance();
			properties=null;
			continue;
		    }
		}
		else if(arg.equals("-ucast")) {
		    factory=new JChannelFactory();
		    properties="UDP(ip_mcast=false):GMS(router_host=" + args[++i] + 
			";router_port=" + new Integer(args[++i]).intValue() +")";
		}
		else if(arg.equals("-help")) {
		    System.out.println("Grid [-type ens | ibus ] " + 
				       "[-ucast <router host> <router port>] [-num <number>] "+
				       "[-sleep <time in ms>] [-log <filename>]");
		    return;
		}
		else if(arg.equals("-num")) {
		    num=new Integer(args[++i]).intValue();
		    continue;
		}		
		else if(arg.equals("-sleep")) {
		    sleeptime=new Integer(args[++i]).intValue();
		    continue;
		}
		else if(arg.equals("-log")) {
		    logfile=args[++i];
		    continue;
		}
	    }
	}
	catch(Exception e) {
	    System.out.println("Grid [-type ens | ibus] [-ucast <router host> <router port>] " +
			       "[-num <number>] [-sleep <time (in ms)>] " +
			       "[-log <filename>]");
	    return;
	}

	grid=new Grid(factory, properties, num, sleeptime, logfile);
	grid.go();
    }





    


    /* --------------- Callbacks --------------- */



    public void SetColor(Integer x, Integer y, Color new_color) {
	synchronized(grid) {
	    Log("SetColor  " + x + "," + y + " (" + new_color.getRed() + "|" + 
		new_color.getGreen() + "|" + new_color.getBlue() + ")\n");
	    grid[x.intValue()][y.intValue()].SetColor(new_color);
	}
    }


    public void Start() {
	if(started)
	    return;
	SetStarted(true);
	checksum_label.setText("Running");
	mythread=new Thread(this);	
	mythread.start();
    }


    void SetStarted(boolean flag) {
	started=flag;
	if(started) {
	    stop_button.setEnabled(true);
	    start_button.setEnabled(false);
	}
	else {
	    stop_button.setEnabled(false);
	    start_button.setEnabled(true);
	}
    }


    public void Stop() {
	if(started) {
	    SetStarted(false);
	    if(mythread != null) {
		try {
		    mythread.join();
		}
		catch(Exception e) {
		    System.err.println(e);
		}
		mythread.stop();
		mythread=null;
	    }	    
	    try {
		dispatcher.CallRemoteMethods(null, "Stop", GroupRequest.GET_ALL, 0);
	    }
	    catch(Exception e) {
		System.err.println(e);
	    }
	}
	/* Give other threads a chance to finish. If we compute the checksum and still
	   receive updates from other threads that haven't finished processing, our checksum
	   will not reflect the current grid ! A better implementation would use acks to
	   make sure all threads have ended ... */
	try {
	    Thread.currentThread().sleep(500);
	}
	catch(Exception e) {
	    System.err.println(e);
	}
	ComputeChecksum();
    }




    public void ComputeChecksum() {
	long   checksum=0;
	Color  col;
	synchronized(grid) {
	    for(int x=0; x < NUM; x++)
		for(int y=0; y < NUM; y++)
		    checksum+=grid[x][y].GetChecksum();
	}
	checksum_label.setText("Checksum: " + checksum);
    }



    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e) {
	setVisible(false);
	channel.Close();
	System.exit(0);
    }
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}


    public void actionPerformed(ActionEvent e) {
	String     command=e.getActionCommand();	
	if(command == "Start")
	    Start();
	else if(command == "Stop")
	    Stop();
	else if(command == "Checksum")
	    //ComputeChecksum();
	    System.out.println(channel.GetView());
	else
	    System.out.println("Unknown action");
    }


}
